@@ -13,14 +13,16 @@
 
 static int find_type PROTO((Node * p, void *closure));
 static int fmt_proc PROTO((Node * p, void *closure));
-static int logfile_write PROTO((char *repository, char *filter,
-			  char *message, FILE * logfp, List * changes));
-static int rcsinfo_proc PROTO((char *repository, char *template));
+static int logfile_write PROTO((const char *repository, const char *filter,
+                                const char *message, FILE * logfp,
+                                List * changes));
+static int rcsinfo_proc PROTO((const char *repository, const char *template));
 static int title_proc PROTO((Node * p, void *closure));
-static int update_logfile_proc PROTO((char *repository, char *filter));
+static int update_logfile_proc PROTO((const char *repository,
+                                      const char *filter));
 static void setup_tmpfile PROTO((FILE * xfp, char *xprefix, List * changes));
-static int editinfo_proc PROTO((char *repository, char *template));
-static int verifymsg_proc PROTO((char *repository, char *script));
+static int editinfo_proc PROTO((const char *repository, const char *template));
+static int verifymsg_proc PROTO((const char *repository, const char *script));
 
 static FILE *fp;
 static char *str_list;
@@ -104,9 +106,8 @@ find_type (p, closure)
     Node *p;
     void *closure;
 {
-    struct logfile_info *li;
+    struct logfile_info *li = p->data;
 
-    li = (struct logfile_info *) p->data;
     if (li->type == type)
 	return (1);
     else
@@ -125,7 +126,7 @@ fmt_proc (p, closure)
 {
     struct logfile_info *li;
 
-    li = (struct logfile_info *) p->data;
+    li = p->data;
     if (li->type == type)
     {
         if (li->tag == NULL
@@ -134,7 +135,7 @@ fmt_proc (p, closure)
 	{
 	    if (col > 0)
 	        (void) fprintf (fp, "\n");
-	    (void) fprintf (fp, "%s", prefix);
+	    (void) fputs (prefix, fp);
 	    col = strlen (prefix);
 	    while (col < 6)
 	    {
@@ -182,9 +183,9 @@ fmt_proc (p, closure)
  */
 void
 do_editor (dir, messagep, repository, changes)
-    char *dir;
+    const char *dir;
     char **messagep;
-    char *repository;
+    const char *repository;
     List *changes;
 {
     static int reuse_log_message = 0;
@@ -195,8 +196,11 @@ do_editor (dir, messagep, repository, changes)
     struct stat pre_stbuf, post_stbuf;
     int retcode = 0;
 
-    assert (current_parsed_root->isremote && !repository
-	    || !current_parsed_root->isremote && repository);
+#ifdef CLIENT_SUPPORT
+    assert (!current_parsed_root->isremote != !repository);
+#else
+    assert (repository);
+#endif
 
     if (noexec || reuse_log_message)
 	return;
@@ -216,7 +220,7 @@ do_editor (dir, messagep, repository, changes)
 
     if (*messagep)
     {
-	(void) fprintf (fp, "%s", *messagep);
+	(void) fputs (*messagep, fp);
 
 	if ((*messagep)[0] == '\0' ||
 	    (*messagep)[strlen (*messagep) - 1] != '\n')
@@ -351,16 +355,16 @@ do_editor (dir, messagep, repository, changes)
     if (fclose (fp) < 0)
 	error (0, errno, "warning: cannot close %s", fname);
 
-    if (pre_stbuf.st_mtime == post_stbuf.st_mtime ||
-	*messagep == NULL ||
-	(*messagep)[0] == '\0' ||
-	strcmp (*messagep, "\n") == 0)
+    /* canonicalize emply messages */
+    if (*messagep != NULL &&
+        (**messagep == '\0' || strcmp (*messagep, "\n") == 0))
+    {
+	free (*messagep);
+	*messagep = NULL;
+    }
+
+    if (pre_stbuf.st_mtime == post_stbuf.st_mtime || *messagep == NULL)
     {
-	if (*messagep)
-	{
-	    free (*messagep);
-	    *messagep = NULL;
-	}
 	for (;;)
 	{
 	    (void) printf ("\nLog message unchanged or not specified\n");
@@ -410,7 +414,7 @@ do_editor (dir, messagep, repository, changes)
 void
 do_verify (messagep, repository)
     char **messagep;
-    char *repository;
+    const char *repository;
 {
     FILE *fp;
     char *fname;
@@ -426,16 +430,16 @@ do_verify (messagep, repository)
 
     /* FIXME? Do we really want to skip this on noexec?  What do we do
        for the other administrative files?  */
-    if (noexec)
+    if (noexec || repository == NULL)
 	return;
 
-    /* If there's no message, then we have nothing to verify.  Can this
-       case happen?  And if so why would we print a message?  */
-    if (*messagep == NULL)
-    {
-	cvs_output ("No message to verify\n", 0);
+    /* Get the name of the verification script to run  */
+
+    if (Parse_Info (CVSROOTADM_VERIFYMSG, repository, verifymsg_proc, 0) > 0)
+	error (1, 0, "Message verification failed");
+
+    if (!verifymsg_script)
 	return;
-    }
 
     /* open a temporary file, write the message to the 
        temp file, and close the file.  */
@@ -443,10 +447,12 @@ do_verify (messagep, repository)
     if ((fp = cvs_temp_file (&fname)) == NULL)
 	error (1, errno, "cannot create temporary file %s", fname);
 
-    fprintf (fp, "%s", *messagep);
-    if ((*messagep)[0] == '\0' ||
+    if (*messagep != NULL)
+	fputs (*messagep, fp);
+    if (*messagep == NULL ||
+	(*messagep)[0] == '\0' ||
 	(*messagep)[strlen (*messagep) - 1] != '\n')
-	(void) fprintf (fp, "%s", "\n");
+	putc ('\n', fp);
     if (fclose (fp) == EOF)
 	error (1, errno, "%s", fname);
 
@@ -463,28 +469,17 @@ do_verify (messagep, repository)
 	sleep_past (pre_stbuf.st_mtime);
     }
 
-    /* Get the name of the verification script to run  */
-
-    if (repository != NULL)
-	(void) Parse_Info (CVSROOTADM_VERIFYMSG, repository, 
-			   verifymsg_proc, 0);
-
-    /* Run the verification script  */
-
-    if (verifymsg_script)
+    run_setup (verifymsg_script);
+    run_arg (fname);
+    if ((retcode = run_exec (RUN_TTY, RUN_TTY, RUN_TTY,
+			     RUN_NORMAL | RUN_SIGIGNORE)) != 0)
     {
-	run_setup (verifymsg_script);
-	run_arg (fname);
-	if ((retcode = run_exec (RUN_TTY, RUN_TTY, RUN_TTY,
-				 RUN_NORMAL | RUN_SIGIGNORE)) != 0)
-	{
-	    /* Since following error() exits, delete the temp file now.  */
-	    if (unlink_file (fname) < 0)
-		error (0, errno, "cannot remove %s", fname);
+	/* Since following error() exits, delete the temp file now.  */
+	if (unlink_file (fname) < 0)
+	    error (0, errno, "cannot remove %s", fname);
 
-	    error (1, retcode == -1 ? errno : 0, 
-		   "Message verification failed");
-	}
+	error (1, retcode == -1 ? errno : 0, 
+	       "Message verification failed");
     }
 
     /* Get the mod time and size of the possibly new log message
@@ -506,27 +501,25 @@ do_verify (messagep, repository)
 	     pre_stbuf.st_size != post_stbuf.st_size)))
     {
 	/* put the entire message back into the *messagep variable */
-	if ( (fp = open_file (fname, "r")) == NULL )
-	    error (1, errno, "cannot open temporary file %s", fname);
 
 	if (*messagep) free (*messagep);
 
 	if (post_stbuf.st_size == 0)
 	    *messagep = NULL;
 	else
-	{
-	    /* On NT, we might read less than st_size bytes,
-	       but we won't read more.  So this works.  */
-	    *messagep = (char *) xmalloc (post_stbuf.st_size + 1);
-	    *messagep[0] = '\0';
-	}
-
-	if (*messagep)
 	{
 	    char *line = NULL;
 	    int line_length;
 	    size_t line_chars_allocated = 0;
-	    char *p = *messagep;
+	    char *p;
+
+	    if ( (fp = open_file (fname, "r")) == NULL )
+		error (1, errno, "cannot open temporary file %s", fname);
+
+	    /* On NT, we might read less than st_size bytes,
+	       but we won't read more.  So this works.  */
+	    p = *messagep = (char *) xmalloc (post_stbuf.st_size + 1);
+	    *messagep[0] = '\0';
 
 	    while (1)
 	    {
@@ -548,9 +541,9 @@ do_verify (messagep, repository)
 		p += line_length;
 	    }
 	    if (line) free (line);
+	    if (fclose (fp) < 0)
+	        error (0, errno, "warning: cannot close %s", fname);
 	}
-	if (fclose (fp) < 0)
-	    error (0, errno, "warning: cannot close %s", fname);
     }
 
     /* Delete the temp file  */
@@ -558,6 +551,8 @@ do_verify (messagep, repository)
     if (unlink_file (fname) < 0)
 	error (0, errno, "cannot remove %s", fname);
     free (fname);
+    free( verifymsg_script );
+    verifymsg_script = NULL;
 }
 
 /*
@@ -568,8 +563,8 @@ do_verify (messagep, repository)
 /* ARGSUSED */
 static int
 rcsinfo_proc (repository, template)
-    char *repository;
-    char *template;
+    const char *repository;
+    const char *template;
 {
     static char *last_template;
     FILE *tfp;
@@ -610,13 +605,13 @@ rcsinfo_proc (repository, template)
  * specified program as standard input.
  */
 static FILE *logfp;
-static char *message;
+static const char *message;
 static List *changes;
 
 void
 Update_Logfile (repository, xmessage, xlogfp, xchanges)
-    char *repository;
-    char *xmessage;
+    const char *repository;
+    const char *xmessage;
     FILE *xlogfp;
     List *xchanges;
 {
@@ -633,17 +628,21 @@ Update_Logfile (repository, xmessage, xlogfp, xchanges)
     (void) Parse_Info (CVSROOTADM_LOGINFO, repository, update_logfile_proc, 1);
 }
 
+
+
 /*
  * callback proc to actually do the logfile write from Update_Logfile
  */
 static int
 update_logfile_proc (repository, filter)
-    char *repository;
-    char *filter;
+    const char *repository;
+    const char *filter;
 {
-    return (logfile_write (repository, filter, message, logfp, changes));
+    return logfile_write (repository, filter, message, logfp, changes);
 }
 
+
+
 /*
  * concatenate each filename/version onto str_list
  */
@@ -652,10 +651,9 @@ title_proc (p, closure)
     Node *p;
     void *closure;
 {
-    struct logfile_info *li;
     char *c;
+    struct logfile_info *li = p->data;
 
-    li = (struct logfile_info *) p->data;
     if (li->type == type)
     {
 	/* Until we decide on the correct logging solution when we add
@@ -712,6 +710,11 @@ title_proc (p, closure)
 		   fields).  This way if future CVS versions add formatting
 		   characters, one can write a loginfo file which at least
 		   won't blow up on an old CVS.  */
+		/* Note that people who have to deal with spaces in file
+		   and directory names are using space to get a known
+		   delimiter for the directory name, so it's probably
+		   not a good idea to ever define that as a formatting
+		   character.  */
 		}
 		if (*(c + 1) != '\0')
 		{
@@ -730,9 +733,9 @@ title_proc (p, closure)
  */
 static int
 logfile_write (repository, filter, message, logfp, changes)
-    char *repository;
-    char *filter;
-    char *message;
+    const char *repository;
+    const char *filter;
+    const char *message;
     FILE *logfp;
     List *changes;
 {
@@ -799,7 +802,7 @@ logfile_write (repository, filter, message, logfp, changes)
     if (fmt_percent)
     {
 	int len;
-	char *srepos;
+	const char *srepos;
 	char *fmt_begin, *fmt_end;	/* beginning and end of the
 					   format string specified in
 					   filter. */
@@ -930,7 +933,7 @@ logfile_write (repository, filter, message, logfp, changes)
     }
 
     setup_tmpfile (pipefp, "", changes);
-    (void) fprintf (pipefp, "Log Message:\n%s\n", message);
+    (void) fprintf (pipefp, "Log Message:\n%s\n", (message) ? message : "");
     if (logfp != (FILE *) 0)
     {
 	(void) fprintf (pipefp, "Status:\n");
@@ -954,8 +957,8 @@ logfile_write (repository, filter, message, logfp, changes)
 /* ARGSUSED */
 static int
 editinfo_proc(repository, editor)
-    char *repository;
-    char *editor;
+    const char *repository;
+    const char *editor;
 {
     /* nothing to do if the last match is the same as this one */
     if (editinfo_editor && strcmp (editinfo_editor, editor) == 0)
@@ -972,8 +975,8 @@ editinfo_proc(repository, editor)
  */
 static int
 verifymsg_proc (repository, script)
-    char *repository;
-    char *script;
+    const char *repository;
+    const char *script;
 {
     if (verifymsg_script && strcmp (verifymsg_script, script) == 0)
 	return (0);
