@@ -18,6 +18,9 @@
 #include "getline.h"
 
 #ifdef MY_NDBM
+# ifndef O_ACCMODE
+#   define O_ACCMODE (O_RDONLY | O_WRONLY | O_RDWR)
+# endif /* defined O_ACCMODE */
 
 static void mydbm_load_file PROTO ((FILE *, List *, char *));
 
@@ -33,7 +36,8 @@ mydbm_open (file, flags, mode)
     FILE *fp;
     DBM *db;
 
-    fp = CVS_FOPEN (file, FOPEN_BINARY_READ);
+    fp = CVS_FOPEN (file, (flags & O_ACCMODE) != O_RDONLY ?
+                                 FOPEN_BINARY_READWRITE : FOPEN_BINARY_READ);
     if (fp == NULL && !(existence_error (errno) && (flags & O_CREAT)))
 	return ((DBM *) 0);
 
@@ -177,12 +181,12 @@ mydbm_store (db, key, value, flags)
     node->type = NDBMNODE;
 
     node->key = xmalloc (key.dsize + 1);
-    strncpy (node->key, key.dptr, key.dsize);
-    node->key[key.dsize] = '\0';
+    *node->key = '\0';
+    strncat (node->key, key.dptr, key.dsize);
 
     node->data = xmalloc (value.dsize + 1);
-    strncpy (node->data, value.dptr, value.dsize);
-    node->data[value.dsize] = '\0';
+    *(char *)node->data = '\0';
+    strncat (node->data, value.dptr, value.dsize);
 
     db->modified = 1;
     if (addnode (db->dbm_list, node) == -1)
