@@ -134,6 +134,25 @@ start_recursion (fileproc, filesdoneproc, direntproc, dirleaveproc, callerdat,
     frame.aflag = aflag;
     frame.locktype = locktype;
     frame.dosrcs = dosrcs;
+
+    /* If our repository_in has a trailing "/.", remove it before storing it
+     * for do_recursion().
+     *
+     * FIXME: This is somewhat of a hack in the sense that many of our callers
+     * painstakingly compute and add the trailing '.' we now remove.
+     */
+    while (repository_in && strlen (repository_in) >= 2
+           && repository_in[strlen (repository_in) - 2] == '/'
+           && repository_in[strlen (repository_in) - 1] == '.')
+    {
+	/* Beware the case where the string is exactly "/." or "//.".
+	 * Paths with a leading "//" are special on some early UNIXes.
+	 */
+	if (strlen (repository_in) == 2 || strlen (repository_in) == 3)
+	    repository_in[strlen (repository_in) - 1] = '\0';
+	else
+	    repository_in[strlen (repository_in) - 2] = '\0';
+    }
     frame.repository = repository_in;
 
     expand_wild (argc, argv, &argc, &argv);
@@ -171,21 +190,24 @@ start_recursion (fileproc, filesdoneproc, direntproc, dirleaveproc, callerdat,
 	    && CVSroot_cmdline == NULL
 	    && current_parsed_root->isremote)
 	{
-	    char *root = Name_Root (NULL, update_dir);
-	    if (root && strcmp (root, current_parsed_root->original) != 0)
-		/* We're skipping this directory because it is for
-		   a different root.  Therefore, we just want to
-		   do the subdirectories only.  Processing files would
-		   cause a working directory from one repository to be
-		   processed against a different repository, which could
-		   cause all kinds of spurious conflicts and such.
-
-		   Question: what about the case of "cvs update foo"
-		   where we process foo/bar and not foo itself?  That
-		   seems to be handled somewhere (else) but why should
-		   it be a separate case?  Needs investigation...  */
-		just_subdirs = 1;
-	    free (root);
+	    cvsroot_t *root = Name_Root (NULL, update_dir);
+	    if (root)
+	    {
+		if (strcmp (root->original, current_parsed_root->original))
+		    /* We're skipping this directory because it is for
+		     * a different root.  Therefore, we just want to
+		     * do the subdirectories only.  Processing files would
+		     * cause a working directory from one repository to be
+		     * processed against a different repository, which could
+		     * cause all kinds of spurious conflicts and such.
+		     *
+		     * Question: what about the case of "cvs update foo"
+		     * where we process foo/bar and not foo itself?  That
+		     * seems to be handled somewhere (else) but why should
+		     * it be a separate case?  Needs investigation...  */
+		    just_subdirs = 1;
+		free_cvsroot_t (root);
+	    }
 	}
 #endif
 
@@ -566,7 +588,7 @@ do_recursion (frame)
      * generating data, to give the buffers a chance to drain to the
      * remote client.  We should not have locks active at this point,
      * but if there are writelocks around, we cannot pause here.  */
-    if (server_active && locktype != CVS_LOCK_NONE)
+    if (server_active && locktype != CVS_LOCK_WRITE)
 	server_pause_check();
 #endif
 
@@ -599,26 +621,30 @@ do_recursion (frame)
 #endif
 	)
     {
-	char *this_root = Name_Root ((char *) NULL, update_dir);
+	cvsroot_t *this_root = Name_Root ((char *) NULL, update_dir);
 	if (this_root != NULL)
 	{
-	    if (findnode (root_directories, this_root) == NULL)
+	    if (findnode (root_directories, this_root->original))
+	    {
+		process_this_directory = !strcmp (current_parsed_root->original,
+						  this_root->original);
+		free_cvsroot_t (this_root);
+	    }
+	    else
 	    {
 		/* Add it to our list. */
 
 		Node *n = getnode ();
 		n->type = NT_UNKNOWN;
-		n->key = xstrdup (this_root);
+		n->key = xstrdup (this_root->original);
+		n->data = this_root;
 
 		if (addnode (root_directories, n))
-		    error (1, 0, "cannot add new CVSROOT %s", this_root);
-	
-	    }
-	
-	    process_this_directory =
-		    (strcmp (current_parsed_root->original, this_root) == 0);
+		    error (1, 0, "cannot add new CVSROOT %s",
+			   this_root->original);
 
-	    free (this_root);
+		process_this_directory = 0;
+	    }
 	}
     }
 
@@ -1047,25 +1073,30 @@ but CVS uses %s for its own purposes; skipping %s directory",
 #endif
 	)
     {
-	char *this_root = Name_Root (dir, update_dir);
+	cvsroot_t *this_root = Name_Root (dir, update_dir);
 	if (this_root != NULL)
 	{
-	    if (findnode (root_directories, this_root) == NULL)
+	    if (findnode (root_directories, this_root->original))
+	    {
+		process_this_directory = !strcmp (current_parsed_root->original,
+						  this_root->original);
+		free_cvsroot_t (this_root);
+	    }
+	    else
 	    {
 		/* Add it to our list. */
 
 		Node *n = getnode ();
 		n->type = NT_UNKNOWN;
-		n->key = xstrdup (this_root);
+		n->key = xstrdup (this_root->original);
+		n->data = this_root;
 
 		if (addnode (root_directories, n))
-		    error (1, 0, "cannot add new CVSROOT %s", this_root);
+		    error (1, 0, "cannot add new CVSROOT %s",
+			   this_root->original);
 
+		process_this_directory = 0;
 	    }
-
-	    process_this_directory = (strcmp (current_parsed_root->original, this_root) == 0);
-
-	    free (this_root);
 	}
     }
 
