@@ -9,9 +9,11 @@
 #include "cvs.h"
 
 #ifdef SERVER_SUPPORT
-static void time_stamp_server PROTO((char *, Vers_TS *, Entnode *));
+static void time_stamp_server PROTO((const char *, Vers_TS *, Entnode *));
 #endif
 
+
+
 /* Fill in and return a Vers_TS structure for the file FINFO.  TAG and
    DATE are from the command line.  */
 
@@ -55,13 +57,13 @@ Version_TS (finfo, options, tag, date, force_tag_match, set_time)
     else
     {
 	p = findnode_fn (finfo->entries, finfo->file);
-	sdtp = (struct stickydirtag *) finfo->entries->list->data; /* list-private */
+	sdtp = finfo->entries->list->data; /* list-private */
     }
 
     entdata = NULL;
     if (p != NULL)
     {
-	entdata = (Entnode *) p->data;
+	entdata = p->data;
 
 	if (entdata->type == ENT_SUBDIR)
 	{
@@ -204,11 +206,10 @@ Version_TS (finfo, options, tag, date, force_tag_match, set_time)
 		struct utimbuf t;
 
 		memset (&t, 0, sizeof (t));
-		t.modtime =
-		    RCS_getrevtime (rcsdata, vers_ts->vn_rcs, 0, 0);
+		t.modtime = RCS_getrevtime (rcsdata, vers_ts->vn_rcs, 0, 0);
 		if (t.modtime != (time_t) -1)
 		{
-		    t.actime = t.modtime;
+		    (void) time (&t.actime);
 
 #ifdef UTIME_EXPECTS_WRITABLE
 		    if (!iswritable (finfo->file))
@@ -226,7 +227,7 @@ Version_TS (finfo, options, tag, date, force_tag_match, set_time)
 		    (void) utime (finfo->file, &t);
 
 #ifdef UTIME_EXPECTS_WRITABLE
-		    if (change_it_back == 1)
+		    if (change_it_back)
 		    {
 			xchmod (finfo->file, 0);
 			change_it_back = 0;
@@ -261,7 +262,7 @@ Version_TS (finfo, options, tag, date, force_tag_match, set_time)
 
 static void
 time_stamp_server (file, vers_ts, entdata)
-    char *file;
+    const char *file;
     Vers_TS *vers_ts;
     Entnode *entdata;
 {
@@ -286,7 +287,7 @@ time_stamp_server (file, vers_ts, entdata)
 	else if (entdata->timestamp
 		 && entdata->timestamp[0] == '=')
 	    mark_unchanged (vers_ts);
-	else if (entdata->timestamp != NULL
+	else if (entdata->timestamp
 		 && (entdata->timestamp[0] == 'M'
 		     || entdata->timestamp[0] == 'D')
 		 && entdata->timestamp[1] == '\0')
@@ -302,7 +303,6 @@ time_stamp_server (file, vers_ts, entdata)
     else
     {
         struct tm *tm_p;
-        struct tm local_tm;
 
 	vers_ts->ts_user = xmalloc (25);
 	/* We want to use the same timestamp format as is stored in the
@@ -313,14 +313,7 @@ time_stamp_server (file, vers_ts, entdata)
 	   stored in local time, and therefore it is not possible to cause
 	   st_mtime to be out of sync by changing the timezone.  */
 	tm_p = gmtime (&sb.st_mtime);
-	if (tm_p)
-	{
-	    memcpy (&local_tm, tm_p, sizeof (local_tm));
-	    cp = asctime (&local_tm);	/* copy in the modify time */
-	}
-	else
-	    cp = ctime (&sb.st_mtime);
-
+	cp = tm_p ? asctime (tm_p) : ctime (&sb.st_mtime);
 	cp[24] = 0;
 	/* Fix non-standard format.  */
 	if (cp[8] == '0') cp[8] = ' ';
@@ -335,7 +328,7 @@ time_stamp_server (file, vers_ts, entdata)
  */
 char *
 time_stamp (file)
-    char *file;
+    const char *file;
 {
     struct stat sb;
     char *cp;
@@ -357,7 +350,6 @@ time_stamp (file)
     if (mtime)
     {
 	struct tm *tm_p;
-        struct tm local_tm;
 	ts = xmalloc (25);
 	/* We want to use the same timestamp format as is stored in the
 	   st_mtime.  For unix (and NT I think) this *must* be universal
@@ -367,14 +359,7 @@ time_stamp (file)
 	   stored in local time, and therefore it is not possible to cause
 	   st_mtime to be out of sync by changing the timezone.  */
 	tm_p = gmtime (&sb.st_mtime);
-	if (tm_p)
-	{
-	    memcpy (&local_tm, tm_p, sizeof (local_tm));
-	    cp = asctime (&local_tm);	/* copy in the modify time */
-	}
-	else
-	    cp = ctime(&sb.st_mtime);
-
+	cp = tm_p ? asctime (tm_p) : ctime (&sb.st_mtime);
 	cp[24] = 0;
 	/* Fix non-standard format.  */
 	if (cp[8] == '0') cp[8] = ' ';
